% Example 2.4 - Nonnegative Matrix Factorization
% In this example, we show how to obtain nonnegative matrix
% factors.

% Exploratory Data Analysis with MATLAB, 2nd Edition
% Wendy L. and Angel R. Martinez and Jeff Solka

% Loads up variable: X, termdoc, docs, and words 
load lsiex
[n,p] = size(termdoc);
% normalize columns to be unit norm
for i = 1:p
	termdoc(:,i) = X(:,i)/norm(X(:,i));
end
% Factor termdoc into nonnegative product U*V where
% U is 6 x 3 and V is 3 x 5
% We will rerun the factorization using the multiplicative 
% update algorithm which is the one we described in our 
% text. We follow this with a calculation of the 
% cosines between the NMF approximation and our query.
  [W, H] = nnmf(termdoc,3,'algorithm','mult');
% Here are our queries:
q1 = [1 0 1 0 0 0];
q2 = [1 0 0 0 0 0];
termdocnmfk = W * H;
% Now we comput the cosine values between our query and
% the columns in our rank k approximation to X obtained vi
% nonnegative factorization
for i = 1:5
	m1 = norm(q1);
	m2 = norm(termdocnmfk(:,i));
	cosq1c(i) = (q1 * termdocnmfk(:,i))/(m1*m2);
	m1 = norm(q2);
	m2 = norm(termdocnmfk(:,i));
	cosq2c(i) = (q2 * termdocnmfk(:,i))/(m1*m2);
	
end
% Using a threshold of .5 the first and fourth document
% match our first query
% 
% cosq1c
% 
% cosq1c =
% 
%     0.7449    0.0000    0.0100    0.7185    0.0056
% Similarly the first and fourth documents match our second 
% query
% cosq2c
% 
% cosq2c =
% 
%     0.5268    0.0000    0.0075    0.5080    0.0043
