% Example 6.9

% Exploratory Data Analysis with MATLAB, 2nd Edition
% Wendy L. and Angel R. Martinez and Jeff Solka

% This shows how to use MBC for density estimation that will then be used
% in a naive Bayes classifier.

% Lets set the model parameters for Class 1.
% First, we will set the means, where each row is a 
% mean vector
MU1 = [3 3;-3 -3];

% Now we set the covariance matrices and put 
% them into a 3-D array
SIGMA1 = cat(3,[1,0;0,1],[.3,0;0,.3]);

% Here we set the mixing proportions with the
% same number of observations in each class.
p1 = ones(1,2)/2;

% This Statistics Toolbox function 
% creates a mixture object.
obj1 = gmdistribution(MU1,SIGMA1,p1);

% Here we draw a random sample of size 1000 from this
% mixture model
X1 = random(obj1, 1000);

% In the following lines of code, we set the parameters
% for the finite mixture for the probability density of
% the second class.

MU2 = [3 -3;-3 3];
SIGMA2 = cat(3,[1,.75;.75,1],[1,0;0,1]);
p2 = ones(1,2)/2;
obj2 = gmdistribution(MU2,SIGMA2,p2);
X2 = random(obj2, 1000);

% Here we plot the observations from this first class in red
% crosses and the second class in green points.
figure
plot(X1(:,1),X1(:,2),'r+')
axis equal
hold on
plot(X2(:,1),X2(:,2),'g.')
axis([-7 7 -7 7])


% Now we are going to estmate the class conditional
% probability densities for each class using our
% model-based clustering toolkit.
[bics1,modelout1,model1,Z1,clabs1] = mbclust(X1,5);

% We next repeat this process for the second classe.
[bics2,modelout2,model2,Z2,clabs2] = mbclust(X2,5);

% The following creates MATLAB objectz that 
% represent the actual density function, so we can 
% use them in Bayes decision rule.
obj1fit = gmdistribution(modelout1.mus',...
    modelout1.vars,modelout1.pies);
obj2fit = gmdistribution(modelout2.mus',...
    modelout2.vars,modelout2.pies);

% Here is the code to plot the discriminant region.
% It will range from -6 to 6 on each axis.
[xx,yy] = meshgrid(-6:.1:6,-6:.1:6);
dom = [xx(:),yy(:)];
% Evaluate each class conditional probability
% at the grid locations, using the pdf function
% in the Statistics Toolbox.
ccp1 = pdf(obj1fit,dom);
ccp2 = pdf(obj2fit,dom);
% Reshape into the grid.
ccp1 = reshape(ccp1,size(xx));
ccp2 = reshape(ccp2,size(xx));

% Since we have equal prior probabilities for
% each class, our decision rule is very simple.
% We just compare the two class-conditional
% probabilities.
ind = ccp1 > ccp2;
figure
surf(xx,yy,+ind)
view([0 90])
axis tight
shading interp



